@@ -1,7 +1,7 @@
 import numpy as np
 import math
 from sklearn.utils.extmath import softmax as softmax_
-from . import hyperopt
+import hyperopt
 import os
 from rs4 import siesta
 
@@ -32,74 +32,78 @@ def render_trial (space, trial, stringfy = False):
 def get_latest_model (path):
     if not os.path.isdir (path) or not os.listdir (path):
         return
-    version = max ([int (ver) for ver in os.listdir (path) if ver.isdigit () and os.path.isdir (os.path.join (path, ver))])    
+    version = max ([int (ver) for ver in os.listdir (path) if ver.isdigit () and os.path.isdir (os.path.join (path, ver))])
     return os.path.join (path, str (version))
- 
- 
+
+def confidence_interval (level, error, n):
+    c = {90: 1.64, 95: 1.96, 98: 2.33, 99: 2.58}[level]
+    return c * math.sqrt((error * (1 - error)) / n)
+
+
 # Model Loaders ------------------------------------------------
 
 class CheckPoint:
     def __init__ (self, net):
         self.net = net
-    
+
     def ftest (self, xs, ys, **kargs):
         return self.net.ftest (self.net.normalize (xs), ys, **kargs)
-            
+
     def predict (self, xs):
         return self.net.run (self.net.logit, x = self.net.normalize (xs)) [0]
-        
+
 
 class SavedModel:
     def __init__ (self, model_root, debug = False):
         self.model_root = model_root
         self.debug = debug
-        self.model_path = get_latest_model (self.model_root)        
+        self.model_path = get_latest_model (self.model_root)
         self.stub = self.create_stub (self.model_path)
-        self.version = os.path.basename (self.model_path)        
-    
+        self.version = os.path.basename (self.model_path)
+
     def create_stub (self, model_path):
         from . import saved_model
         return saved_model.load (model_path)
-                
+
     def predict (self, x, **kargs):
         y = self.stub.run (x, **kargs)[0]
         return y.astype (np.float64)
 
-    
-class TFLite (SavedModel):    
+
+class TFLite (SavedModel):
     def __init__ (self, model_root, model_file = "model.tflite", debug = False):
         self.model_file = model_file
         SavedModel.__init__ (self, model_root, debug)
-    
+
     def create_stub (self, model_path):
         from . import tflite
-        
+
         return tflite.load (
             os.path.join (model_path, self.model_file),
             (128, 128)
         )
-    
-    def predict (self, x, **kargs):   
+
+    def predict (self, x, **kargs):
         ys = []
-        for x_ in x:     
+        for x_ in x:
             ys.append (self.stub.run (x_, **kargs)[0].astype (np.float64))
-        return np.array (ys)    
+        return np.array (ys)
 
 
 class TFServer:
     def __init__ (self, endpoint, alias, debug = False):
         from tfserver import cli
-                
+
         self.endpoint = endpoint
         self.alias = alias
         self.debug = debug
-        
+
         self.stub = cli.Server (endpoint)
         api = siesta.API (endpoint)
         resp = api.model (self.alias).version.get ()
-        self.version = resp.data.version         
-        
+        self.version = resp.data.version
+
     def predict (self, x, **kargs):
         resp = self.stub.predict (self.alias, 'predict', x = x, **kargs)
         return resp.y
-            
\ No newline at end of file
+
